package org.jboss.security.plugins;

import java.beans.PropertyEditorManager;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;
import javax.naming.CommunicationException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.OperationNotSupportedException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;
import javax.naming.spi.ObjectFactory;
import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.jacc.PolicyContext;
import org.jboss.logging.Logger;
import org.jboss.security.AuthenticationManager;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SecurityDomain;
import org.jboss.security.SecurityProxyFactory;
import org.jboss.security.SubjectSecurityProxyFactory;
import org.jboss.security.auth.callback.SecurityAssociationHandler;
import org.jboss.security.jacc.SubjectPolicyContextHandler;
import org.jboss.security.plugins.JaasSecurityManagerServiceMBean;
import org.jboss.security.plugins.SecurityDomainContext;
import org.jboss.security.plugins.SubjectActions;
import org.jboss.security.propertyeditor.PrincipalEditor;
import org.jboss.security.propertyeditor.SecurityDomainEditor;
import org.jboss.system.ServiceMBeanSupport;
import org.jboss.util.CachePolicy;
import org.jboss.util.TimedCachePolicy;

public class JaasSecurityManagerService extends ServiceMBeanSupport implements JaasSecurityManagerServiceMBean {

   private static final String SECURITY_MGR_PATH = "java:/jaas";
   private static final String DEFAULT_CACHE_POLICY_PATH = "java:/timedCacheFactory";
   private static Logger log = Logger.getLogger(JaasSecurityManagerService.class);
   private static String securityMgrClassName = "org.jboss.security.plugins.JaasSecurityManager";
   private static Class securityMgrClass;
   private static String callbackHandlerClassName = "org.jboss.security.auth.callback.SecurityAssociationHandler";
   private static Class callbackHandlerClass = SecurityAssociationHandler.class;
   private static String cacheJndiName = "java:/timedCacheFactory";
   private static int defaultCacheTimeout = 1800;
   private static int defaultCacheResolution = 60;
   private static String securityProxyFactoryClassName = "org.jboss.security.SubjectSecurityProxyFactory";
   private static Class securityProxyFactoryClass = SubjectSecurityProxyFactory.class;
   private static Hashtable securityDomainCtxMap = new Hashtable();
   private static NameParser parser;
   private boolean serverMode = true;
   private static String defaultUnauthenticatedPrincipal = "Unauthenticated Principal";


   public boolean getServerMode() {
      return this.serverMode;
   }

   public void setServerMode(boolean mode) {
      this.serverMode = mode;
   }

   public String getSecurityManagerClassName() {
      return securityMgrClassName;
   }

   public void setSecurityManagerClassName(String className) throws ClassNotFoundException, ClassCastException {
      securityMgrClassName = className;
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      securityMgrClass = loader.loadClass(securityMgrClassName);
      if(!AuthenticationManager.class.isAssignableFrom(securityMgrClass)) {
         throw new ClassCastException(securityMgrClass + " does not implement " + AuthenticationManager.class);
      }
   }

   public String getSecurityProxyFactoryClassName() {
      return securityProxyFactoryClassName;
   }

   public void setSecurityProxyFactoryClassName(String className) throws ClassNotFoundException {
      securityProxyFactoryClassName = className;
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      securityProxyFactoryClass = loader.loadClass(securityProxyFactoryClassName);
   }

   public String getCallbackHandlerClassName() {
      return callbackHandlerClassName;
   }

   public void setCallbackHandlerClassName(String className) throws ClassNotFoundException {
      callbackHandlerClassName = className;
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      callbackHandlerClass = loader.loadClass(callbackHandlerClassName);
   }

   public String getAuthenticationCacheJndiName() {
      return cacheJndiName;
   }

   public void setAuthenticationCacheJndiName(String jndiName) {
      cacheJndiName = jndiName;
   }

   public int getDefaultCacheTimeout() {
      return defaultCacheTimeout;
   }

   public void setDefaultCacheTimeout(int timeoutInSecs) {
      defaultCacheTimeout = timeoutInSecs;
   }

   public int getDefaultCacheResolution() {
      return defaultCacheResolution;
   }

   public void setDefaultCacheResolution(int resInSecs) {
      defaultCacheResolution = resInSecs;
   }

   public void setCacheTimeout(String securityDomain, int timeoutInSecs, int resInSecs) {
      CachePolicy cache = getCachePolicy(securityDomain);
      if(cache != null && cache instanceof TimedCachePolicy) {
         TimedCachePolicy tcp = (TimedCachePolicy)cache;
         synchronized(tcp) {
            tcp.setDefaultLifetime(timeoutInSecs);
            tcp.setResolution(resInSecs);
         }
      } else {
         log.warn("Failed to find cache policy for securityDomain=\'" + securityDomain + "\'");
      }

   }

   public void flushAuthenticationCache(String securityDomain) {
      CachePolicy cache = getCachePolicy(securityDomain);
      if(cache != null) {
         cache.flush();
      } else {
         log.warn("Failed to find cache policy for securityDomain=\'" + securityDomain + "\'");
      }

   }

   public void flushAuthenticationCache(String securityDomain, Principal user) {
      CachePolicy cache = getCachePolicy(securityDomain);
      if(cache != null) {
         cache.remove(user);
      } else {
         log.warn("Failed to find cache policy for securityDomain=\'" + securityDomain + "\'");
      }

   }

   public List getAuthenticationCachePrincipals(String securityDomain) {
      CachePolicy cache = getCachePolicy(securityDomain);
      List validPrincipals = null;
      if(cache instanceof TimedCachePolicy) {
         TimedCachePolicy tcache = (TimedCachePolicy)cache;
         validPrincipals = tcache.getValidKeys();
      }

      return validPrincipals;
   }

   public boolean isValid(String securityDomain, Principal principal, Object credential) {
      boolean isValid = false;

      try {
         SecurityDomainContext e = lookupSecurityDomain(securityDomain);
         isValid = e.getSecurityManager().isValid(principal, credential, (Subject)null);
      } catch (NamingException var6) {
         log.debug("isValid(" + securityDomain + ") failed", var6);
      }

      return isValid;
   }

   public Principal getPrincipal(String securityDomain, Principal principal) {
      Principal realmPrincipal = null;

      try {
         SecurityDomainContext e = lookupSecurityDomain(securityDomain);
         realmPrincipal = e.getRealmMapping().getPrincipal(principal);
      } catch (NamingException var5) {
         log.debug("getPrincipal(" + securityDomain + ") failed", var5);
      }

      return realmPrincipal;
   }

   public boolean doesUserHaveRole(String securityDomain, Principal principal, Object credential, Set roles) {
      boolean doesUserHaveRole = false;

      try {
         SecurityDomainContext e = lookupSecurityDomain(securityDomain);
         Subject subject = new Subject();
         e.getSecurityManager().isValid(principal, credential, subject);
         SubjectActions.pushSubjectContext(principal, credential, subject);
         doesUserHaveRole = e.getRealmMapping().doesUserHaveRole(principal, roles);
         SubjectActions.popSubjectContext();
      } catch (NamingException var8) {
         log.debug("doesUserHaveRole(" + securityDomain + ") failed", var8);
      }

      return doesUserHaveRole;
   }

   public Set getUserRoles(String securityDomain, Principal principal, Object credential) {
      Set userRoles = null;

      try {
         SecurityDomainContext e = lookupSecurityDomain(securityDomain);
         e.getSecurityManager().isValid(principal, credential, (Subject)null);
         userRoles = e.getRealmMapping().getUserRoles(principal);
      } catch (NamingException var6) {
         log.debug("getUserRoles(" + securityDomain + ") failed", var6);
      }

      return userRoles;
   }

   protected void startService() throws Exception {
      if(this.serverMode) {
         SecurityAssociation.setServer();
      }

      SubjectPolicyContextHandler handler = new SubjectPolicyContextHandler();
      PolicyContext.registerHandler("javax.security.auth.Subject.container", handler, true);
      InitialContext ctx = new InitialContext();
      parser = ctx.getNameParser("");
      StringRefAddr refAddr = new StringRefAddr("nns", "JSM");
      String factoryName = JaasSecurityManagerService.SecurityDomainObjectFactory.class.getName();
      Reference ref = new Reference("javax.naming.Context", refAddr, factoryName, (String)null);
      ctx.rebind("java:/jaas", ref);
      log.debug("securityMgrCtxPath=java:/jaas");
      refAddr = new StringRefAddr("nns", "JSMCachePolicy");
      factoryName = JaasSecurityManagerService.DefaultCacheObjectFactory.class.getName();
      ref = new Reference("javax.naming.Context", refAddr, factoryName, (String)null);
      ctx.rebind("java:/timedCacheFactory", ref);
      log.debug("cachePolicyCtxPath=" + cacheJndiName);
      SecurityProxyFactory proxyFactory = (SecurityProxyFactory)securityProxyFactoryClass.newInstance();
      ctx.bind("java:/SecurityProxyFactory", proxyFactory);
      log.debug("SecurityProxyFactory=" + proxyFactory);
      PropertyEditorManager.registerEditor(Principal.class, PrincipalEditor.class);
      PropertyEditorManager.registerEditor(SecurityDomain.class, SecurityDomainEditor.class);
      log.debug("Registered PrincipalEditor, SecurityDomainEditor");
      log.debug("ServerMode=" + this.serverMode);
      log.debug("SecurityMgrClass=" + securityMgrClass);
      log.debug("CallbackHandlerClass=" + callbackHandlerClass);
   }

   protected void stopService() throws Exception {
      InitialContext ic = new InitialContext();

      try {
         ic.unbind("java:/jaas");
      } catch (CommunicationException var6) {
         ;
      } finally {
         ic.close();
      }

   }

   public void registerSecurityDomain(String securityDomain, SecurityDomain instance) {
      log.debug("Added " + securityDomain + ", " + instance + " to map");
      CachePolicy authCache = lookupCachePolicy(securityDomain);
      SecurityDomainContext sdc = new SecurityDomainContext(instance, authCache);
      securityDomainCtxMap.put(securityDomain, sdc);
      setSecurityDomainCache(instance, authCache);
   }

   private static CachePolicy getCachePolicy(String securityDomain) {
      if(securityDomain.startsWith("java:/jaas")) {
         securityDomain = securityDomain.substring("java:/jaas".length() + 1);
      }

      CachePolicy cache = null;

      try {
         SecurityDomainContext e = lookupSecurityDomain(securityDomain);
         if(e != null) {
            cache = e.getAuthenticationCache();
         }
      } catch (NamingException var3) {
         log.debug("getCachePolicy(" + securityDomain + ") failure", var3);
      }

      return cache;
   }

   private static CachePolicy lookupCachePolicy(String securityDomain) {
      CachePolicy authCache = null;
      String domainCachePath = cacheJndiName + '/' + securityDomain;

      try {
         InitialContext e = new InitialContext();
         authCache = (CachePolicy)e.lookup(domainCachePath);
      } catch (Exception var6) {
         try {
            InitialContext e2 = new InitialContext();
            authCache = (CachePolicy)e2.lookup(cacheJndiName);
         } catch (Exception var5) {
            log.warn("Failed to locate auth CachePolicy at: " + cacheJndiName + " for securityDomain=" + securityDomain);
         }
      }

      return authCache;
   }

   private static void setSecurityDomainCache(AuthenticationManager securityMgr, CachePolicy cachePolicy) {
      try {
         Class[] e2 = new Class[]{CachePolicy.class};
         Method m = securityMgrClass.getMethod("setCachePolicy", e2);
         Object[] setCachePolicyArgs = new Object[]{cachePolicy};
         m.invoke(securityMgr, setCachePolicyArgs);
         log.debug("setCachePolicy, c=" + setCachePolicyArgs[0]);
      } catch (Exception var5) {
         log.debug("setCachePolicy failed", var5);
      }

   }

   private static SecurityDomainContext lookupSecurityDomain(String securityDomain) throws NamingException {
      SecurityDomainContext securityDomainCtx = (SecurityDomainContext)securityDomainCtxMap.get(securityDomain);
      if(securityDomainCtx == null) {
         securityDomainCtx = newSecurityDomainCtx(securityDomain);
         securityDomainCtxMap.put(securityDomain, securityDomainCtx);
         log.debug("Added " + securityDomain + ", " + securityDomainCtx + " to map");
      }

      return securityDomainCtx;
   }

   private static SecurityDomainContext newSecurityDomainCtx(String securityDomain) throws NamingException {
      SecurityDomainContext sdc = null;

      try {
         Class[] e2 = new Class[]{String.class, CallbackHandler.class};
         Constructor msg1 = securityMgrClass.getConstructor(e2);
         CallbackHandler ne1 = (CallbackHandler)callbackHandlerClass.newInstance();
         Object[] args = new Object[]{securityDomain, ne1};
         AuthenticationManager securityMgr = (AuthenticationManager)msg1.newInstance(args);
         log.debug("Created securityMgr=" + securityMgr);
         CachePolicy cachePolicy = lookupCachePolicy(securityDomain);
         sdc = new SecurityDomainContext(securityMgr, cachePolicy);
         setSecurityDomainCache(securityMgr, cachePolicy);
         return sdc;
      } catch (Exception var8) {
         String msg = "Failed to create sec mgr(\'" + securityDomain + "\'), securityMgrClass=" + securityMgrClass + ", callbackHandlerClass=" + callbackHandlerClass;
         log.error(msg, var8);
         NamingException ne = new NamingException(msg);
         ne.setRootCause(var8);
         throw ne;
      }
   }

   public String getDefaultUnauthenticatedPrincipal() {
      return defaultUnauthenticatedPrincipal;
   }

   public void setDefaultUnauthenticatedPrincipal(String principal) {
      defaultUnauthenticatedPrincipal = principal;
   }


   public static class DefaultCacheObjectFactory implements InvocationHandler, ObjectFactory {

      public Object getObjectInstance(Object obj, Name name, Context nameCtx, Hashtable environment) throws Exception {
         ClassLoader loader = Thread.currentThread().getContextClassLoader();
         Class[] interfaces = new Class[]{Context.class};
         Context ctx = (Context)Proxy.newProxyInstance(loader, interfaces, this);
         return ctx;
      }

      public Object invoke(Object obj, Method method, Object[] args) throws Throwable {
         TimedCachePolicy cachePolicy = new TimedCachePolicy(JaasSecurityManagerService.defaultCacheTimeout, true, JaasSecurityManagerService.defaultCacheResolution);
         cachePolicy.create();
         cachePolicy.start();
         return cachePolicy;
      }
   }

   static class DomainEnumeration implements NamingEnumeration {

      Enumeration domains;
      Hashtable ctxMap;


      DomainEnumeration(Enumeration domains, Hashtable ctxMap) {
         this.domains = domains;
         this.ctxMap = ctxMap;
      }

      public void close() {
      }

      public boolean hasMoreElements() {
         return this.domains.hasMoreElements();
      }

      public boolean hasMore() {
         return this.domains.hasMoreElements();
      }

      public Object next() {
         String name = (String)this.domains.nextElement();
         Object value = this.ctxMap.get(name);
         String className = value.getClass().getName();
         NameClassPair pair = new NameClassPair(name, className);
         return pair;
      }

      public Object nextElement() {
         return this.domains.nextElement();
      }
   }

   public static class SecurityDomainObjectFactory implements InvocationHandler, ObjectFactory {

      public Object getObjectInstance(Object obj, Name name, Context nameCtx, Hashtable environment) throws Exception {
         ClassLoader loader = SubjectActions.getContextClassLoader();
         Class[] interfaces = new Class[]{Context.class};
         Context ctx = (Context)Proxy.newProxyInstance(loader, interfaces, this);
         return ctx;
      }

      public Object invoke(Object obj, Method method, Object[] args) throws Throwable {
         String methodName = method.getName();
         if(methodName.equals("toString")) {
            return "java:/jaas Context proxy";
         } else if(methodName.equals("list")) {
            return new JaasSecurityManagerService.DomainEnumeration(JaasSecurityManagerService.securityDomainCtxMap.keys(), JaasSecurityManagerService.securityDomainCtxMap);
         } else if(!methodName.equals("lookup")) {
            throw new OperationNotSupportedException("Only lookup is supported, op=" + method);
         } else {
            String securityDomain = null;
            Name name = null;
            if(args[0] instanceof String) {
               name = JaasSecurityManagerService.parser.parse((String)args[0]);
            } else {
               name = (Name)args[0];
            }

            securityDomain = name.get(0);
            SecurityDomainContext securityDomainCtx = JaasSecurityManagerService.lookupSecurityDomain(securityDomain);
            Object binding = securityDomainCtx.getSecurityManager();
            if(name.size() == 2) {
               String request = name.get(1);
               binding = securityDomainCtx.lookup(request);
            }

            return binding;
         }
      }
   }
}
